[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Ownership client library for Python

[![API Version](https://img.shields.io/badge/api-v1.1.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetOwnership/1.0.15)](https://pypi.org/project/fds.sdk.FactSetOwnership/v/1.0.15)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

FactSet's Fund Ownership API gives access to both **Holdings** and **Holders** data.<p> Factset's Holdings endpoints gives access to all the underlying securities and their position detils held within a given fund. Fund Types supported include Open-End Mutual Funds, Closed-end Mutual Funds, and Exchange Traded Funds. Security Holders information retrieves all \"holder types\" and their positions across institutions, funds, insiders, and stakeholders.</p><p>The FactSet Ownership and Mutual Funds database collects global equity ownership data for approximately 50,000 institutions, 60,000 unique Mutual Fund portfolios, and 400,000 Insider/Stake holders from around 110 countries.  For more details review our [Data Collection](https://my.apps.factset.com/oa/cms/oaAttachment/87e162be-f2d1-4f40-a85b-bfb1b020d270/20079) methodology. </p>


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.1.0
- SDK version: 1.0.15
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetOwnership==1.0.15
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetOwnership==1.0.15
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetOwnership
from fds.sdk.FactSetOwnership.api import fund_holdings_api
from fds.sdk.FactSetOwnership.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetOwnership.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetOwnership.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetOwnership.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = fund_holdings_api.FundHoldingsApi(api_client)
    ids = ["VTI-USA"] # [str] | List of requested fund identifiers. <p>***ids limit** =  10 per request*</p>
    date = "2019-12-31" # str | Date of holdings expressed in YYYY-MM-DD format. The fund-holdings endpoint will default to latest month-end close. (optional)
    topn = "5" # str | Limits number of holdings or holders displayed by the top *n* securities based on positions Market Value. Default is ALL, otherwise use number to limit number. (optional) if omitted the server will use the default value of "ALL"
    asset_type = "EQ" # str | Filter holdings by the following major asset classes -   * **EQ**  = Equity   * **FI** = Fixed Income   * **ALL** = ALL  (optional) if omitted the server will use the default value of "EQ"
    currency = "USD" # str | Currency code for adjusting prices. Default is Local. For a list of currency ISO codes, visit [Online Assistant Page 1470](https://oa.apps.factset.com/pages/1470). (optional)

    try:
        # Get underlying holdings information for a requested fund identifer.
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_ownership_holdings(ids, date=date, topn=topn, asset_type=asset_type, currency=currency)

        pprint(api_response)
    except fds.sdk.FactSetOwnership.ApiException as e:
        print("Exception when calling FundHoldingsApi->get_ownership_holdings: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Get underlying holdings information for a requested fund identifer.
    #     api_response, http_status_code, response_headers = api_instance.get_ownership_holdings_with_http_info(ids, date=date, topn=topn, asset_type=asset_type, currency=currency)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetOwnership.ApiException as e:
    #     print("Exception when calling FundHoldingsApi->get_ownership_holdings: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Get underlying holdings information for a requested fund identifer.
    #     async_result = api_instance.get_ownership_holdings_async(ids, date=date, topn=topn, asset_type=asset_type, currency=currency)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetOwnership.ApiException as e:
    #     print("Exception when calling FundHoldingsApi->get_ownership_holdings: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Get underlying holdings information for a requested fund identifer.
    #     async_result = api_instance.get_ownership_holdings_with_http_info_async(ids, date=date, topn=topn, asset_type=asset_type, currency=currency)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetOwnership.ApiException as e:
    #     print("Exception when calling FundHoldingsApi->get_ownership_holdings: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetOwnership

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetOwnership.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetOwnership

configuration = fds.sdk.FactSetOwnership.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetOwnership

configuration = fds.sdk.FactSetOwnership.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetOwnership

configuration = fds.sdk.FactSetOwnership.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*FundHoldingsApi* | [**get_ownership_holdings**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/FundHoldingsApi.md#get_ownership_holdings) | **GET** /factset-ownership/v1/fund-holdings | Get underlying holdings information for a requested fund identifer.
*FundHoldingsApi* | [**post_ownership_holdings**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/FundHoldingsApi.md#post_ownership_holdings) | **POST** /factset-ownership/v1/fund-holdings | Get holdings for a list of funds.
*SecurityHoldersApi* | [**get_security_holders**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/SecurityHoldersApi.md#get_security_holders) | **GET** /factset-ownership/v1/security-holders | Get security ownership data for requested security identifers.
*SecurityHoldersApi* | [**post_security_holders**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/SecurityHoldersApi.md#post_security_holders) | **POST** /factset-ownership/v1/security-holders | Get security ownership data for a list of requested securities.


## Documentation For Models

 - [AssetType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/AssetType.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/ErrorResponseSubErrors.md)
 - [FundHolding](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/FundHolding.md)
 - [FundHoldingsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/FundHoldingsRequest.md)
 - [FundHoldingsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/FundHoldingsResponse.md)
 - [HolderType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/HolderType.md)
 - [IdFundHoldings](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/IdFundHoldings.md)
 - [IdHolders](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/IdHolders.md)
 - [SecurityHolders](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/SecurityHolders.md)
 - [SecurityHoldersRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/SecurityHoldersRequest.md)
 - [SecurityHoldersResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetOwnership/v1/docs/SecurityHoldersResponse.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetOwnership.apis and fds.sdk.FactSetOwnership.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetOwnership.api.default_api import DefaultApi`
- `from fds.sdk.FactSetOwnership.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetOwnership
from fds.sdk.FactSetOwnership.apis import *
from fds.sdk.FactSetOwnership.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

